{---------------------------------------------------------------------------}
{ Toolkit for Windows v1.0  - SToMP version             20 Sep  1993   AYKC }
{   TkStomp.pas - interface for Turbo Pascal for Windows programs to use    }
{                 the graphic DLL TkGraph.DLL                               }
{   email: aykc1@phx.cam.ac.uk                                              }
{---------------------------------------------------------------------------}

UNIT TkStomp;
{$H-}
{$DEFINE DOUBLE}

{ Set compiler flags }
{$IFNDEF REAL} {$N+}
  {$IFNDEF SINGLE} {$IFNDEF DOUBLE} {$IFNDEF EXTENDED}
    {$IFDEF CPU87} {$DEFINE DOUBLE} {$ELSE} {$DEFINE REAL} {$N-} {$ENDIF}
  {$ENDIF} {$ENDIF} {$ENDIF}
{$ENDIF}

INTERFACE

{The real data type used in graphics DLL is DOUBLE}
{ NEW 17 Apr 2000 AYKC }
USES Windows, Messages, SysUtils, Classes, Dialogs;

{--$IFDEF VER70
  USES WinProcs, WinTypes, Objects;
$ELSE
  USES WinProcs, WinTypes, WObjects;
$ENDIF}

CONST GRAPHDLL = 'TkGrpW32';

{-- Real type --------------------------------------------------------------}
TYPE  { Define real type 'TkReal' }
{$IFDEF REAL}     TkReal = real;     {$ENDIF}
{$IFDEF SINGLE}   TkReal = single;   {$ENDIF}
{$IFDEF DOUBLE}   TkReal = double;   {$ENDIF}
{$IFDEF EXTENDED} TkReal = extended; {$ENDIF}

CONST { Ranges of reals. machine accuracy }
{$IFDEF REAL} MAXREAL = 1.5E+38;
  {$IFOPT N+} MINREAL = 2.9E-39; {$ELSE} MINREAL = 1.0E-38; {$ENDIF}
  EPSILON = 1.9E-12;
{$ENDIF}
{$IFDEF SINGLE}
    MAXREAL = 3.4E+38; MINREAL = 1.5E-45; EPSILON = 6.0E-8;
{$ENDIF}
{$IFDEF DOUBLE}
    MAXREAL = 1.7E+308; MINREAL = 5.0E-324; EPSILON = 1.2E-16;
{$ENDIF}
{$IFDEF EXTENDED}
    MAXREAL = 1.1E+4932; MINREAL = 3.4E-4932; EPSILON = 5.5E-20;
{$ENDIF}

{-- References -------------------------------------------------------------}
TYPE String1 = string[1];
     String2 = string[2];
     String3 = string[3];
     String4 = string[4];
     String5 = string[5];
     String6 = string[6];
TYPE PArray = ^String1;
     PkGraph = ^String2;
     PODEFuncs = ^String3;
     Coordptr  = ^String4;
     Funcptr   = ^String5;
     Realptr = ^String6;
     TkBitmap  = HBitmap;
TYPE Arrayptr = ^ Datarray { reference a dynamic array, used in Rkf only };
     Datarray = ARRAY [1..6550] OF TkReal;

{-- Definitions for colors -------------------------------------------------}
TYPE TkColor = Longint; 
CONST NumColors = 16; { no. of entries in Colour table TkColorSet }

VAR TkBlack, TkBlue, TkGreen, TkCyan, TkRed, TkMagenta,
    TkBrown, TkLightGray, TkDarkGray, TkLightBlue,
    TkLightGreen, TkLightCyan, TkLightRed, TkLightMagenta,
    TkYellow, TkWhite: TkColor;
VAR TkColorSet : ARRAY [0..NumColors-1] OF TkColor ABSOLUTE TkBlack;

{-- Special types for TkDraw -----------------------------------------------} 
TYPE Drawobjectoption = (CONTOURS, CONTOURS2D, CONTOURS3D, WIREFRAME,
       SURFACE, SOLID);
     Axismodeoption = (PRESET, RESCALE, DRAWAXES, AUTOAXES);

{-- The following constants are used by drawing routines --------------------}
CONST { redrawing modes }
  Foreground_mode = 1; { draw to window only }
  AutoRedraw_mode = 2; { Synchronous drawing }
  Background_mode = 3; { draw to memory first, sent to window later }
  DelayRedraw_mode = 4; { ditto, but use only 1 bitmap }
  hasBorder_mode = 64;  { extended style used in TkGraph }
  hideFirst_mode = 128; { extended style used in TkGraph }

CONST { Type of DCs in toolkit redrawing system }
  Window_DC   = 1;  { DC for actual window }
  Drawing_DC  = 2;  { DC for internal drawing }
  Backup_DC   = 3;  { DC for backing up screen image }

CONST { line style constants }
  Solid_line  = 1; 
  Dotted_line = 2;
  Dashed_line = 3;
  Null_Line   = 4;
  Gray_Line   = 5;

CONST { pen mode constants }
  Actual_mode = 1;
  Invert_mode = 2;
  AND_mode    = 3;
  OR_mode     = 4;
  EOR_mode    = 5;

CONST { text face constants }
  Roman_face  = 1;
  Helv_face   = 2;
  System_face = 3;
  Symbol_face = 4;
  Fixed_face  = 5;

CONST { text style constants }
  Plain_style       = 1;
  Bold_style        = 2;
  Italic_style      = 3;
  BoldnItalic_style = 4;

TYPE { store information about pen creation }
  PenInfoData = RECORD
    PColor, FColor: TkColor;
    Pstyle, Psize: integer
  END;

CONST { text orientation - FontInfoData::FontOrientation }
  Horizontal_text    = 1; { normal x axis }
  VerticalUp_text    = 2; { normal y axis }
  InvHorizontal_text = 3;
  VerticalDown_text  = 4;

CONST { text justified modes - TextInfoData::TextJustifiedMode }
  Left_Justified     =  1;
  Right_Justified    =  2;
  Centre_Justified   =  3;

TYPE { store information about font creation }
  FontInfoData = RECORD
    Fface, Fstyle, Fsize, FOrientation: integer
  END;

TYPE { store information about background }
  BkgndInfoData = RECORD
    Bcolor: TkColor;
    Bwritemode: integer;
  END;

CONST { default drawing settings options }
  Registry_settings = 0;
  SToMPLight_settings = 1;
  SToMPDark_settings = 2;
  OldIBM_settings = 3;

TYPE { DCSettings controls various default settings of a DC. }
  PDCSettings = ^DCSettings;
  DCSettings = RECORD
    TextColor: TkColor;       { default text color }
    DrawColor: TkColor;       { default drawing color }
    AxisColor: TkColor;       { default axis color }
    PipColor: TkColor;        { default pip color }
    BkgndInfo: BkgndInfoData; { default background color and writemode }
    SysCharInfo: FontInfoData;{ default font }
    TextJustified: integer;   { default text justification }
    Drawingmode: integer;     { toolkit redrawing mode }
    OptionID: integer;
  END;

CONST { mouse shape constants }
  Arrow_shape     = 1;
  Hourglass_shape = 2;
  Cross_shape     = 3;
  Hand_shape      = 4;
  IBeam_shape     = 5;
  User_shape      = 6;
  Dot_shape       = 7;
  NumMouseShapes  = 7;

CONST { Set scales modes used by SetScalesMode }
  TrueRatio_scaling = 1; { both 2D and 3D, always true aspect ratio }
  FillWindow_2D_scaling = 2; { 2D case, always fit the view rect }
  BoundCube_3D_scaling = 3; { 3D case, fit space within ranges into a cube }
  Auto_scaling = 4; { 2D case, choose a method automatically }
    { 3D scaling also affected by Custom3DZoom: percentage of graph window,
      and position of ProjectedOrigin, also percentage of graph window
      measured from top left. }

CONST { Axes types used by AxesType }
  Open_axes = 1;   { axes meet at origin }
  Corner_axes = 2; { where origin is at bottom-left corner }
  Boxed_axes = 3; { or closed axes }

CONST { label position reference used by xAxisTitle, etc. }
  LoEnd_Ref = 1;
  Mid_Ref   = 2;
  HiEnd_Ref = 3;

CONST { label types used by axes labelling routines, etc. }
  XAxis_label = 1;
  YAxis_label = 2;
  ZAxis_label = 3;
  XAxis_title = 4;
  YAxis_title = 5;
  ZAxis_title = 6;
  Graph_title = 7;
  Data_label  = 8;

CONST { label position adjustment for end of axis labelling used by Endmode }
  Auto_ends = 1;
  Invisible_ends = 2;

TYPE { *** NEW: for data/axis label/title }
  PLabelSettings = ^LabelSettings;
  LabelSettings = RECORD
    AutoPosition     : Wordbool;
    LabelFace        : integer;
    LabelStyle       : integer;
    LabelSize        : integer;
    LabelOrientation : integer;
    LabelJustified   : integer;
    LabelColor       : TkColor;
    Labeldx, Labeldy : integer;
    TitleRef         : integer; { only for titles }
    AxisEndmode      : integer; { only for axes }
  END;

CONST { pip modes used in PipSettings.PipMode }
  Outside_pips = 0; { alternative for corner_axes and boxed axes }
  Inside_pips = 1; { standard for corner_axes and boxed axes }
  Centre_pips = 2; { standard for open_axes }
  Invisible_pips = 3; { hide pips }

TYPE
  PipSettings = RECORD
    PipLength: integer; { in pixels }
    SubPipLength: integer; { in pixels }
    PipMode: integer;
  END;

CONST { color constants used in TkSetGraphColor, etc. }
  Axis_color = 1;       { color for all axes }
  Pip_color  = 2;       { color for all pips }
  AxisLabel_color = 3;  { (set) color for all axis labels;
                         (get) color for x-axis label }
  Annotation_color = 4; { current text color }
  Title_color = 5;      { (set) color for all titles;
                          (get) color for graph title }
  GraphBkgnd_Color = 6; { current background color }
  Line_color = 7;       { current line color }
  GraphLine1_color = 8; { SToMP style line1 color }
  GraphLine2_color = 9; { SToMP style line2 color }
  GraphLine3_color = 10; { SToMP style line3 color }
  GraphLine4_color = 11; { SToMP style line4 color }
  { new for grid lines }
  GridLine1_color = 12; { SToMP grid line1 color }
  GridLine2_color = 13; { SToMP grid line2 color }
  Point_color = 14;     { for draw_data family }
  TickLabel_color = 15; { alternatively way to get/set this color quickly }
  XAxisTitle_color = 16; { ditto }
  YAxisTitle_color = 17; { ditto }
  ZAxisTitle_color = 18; { ditto }
  AxisPip_color = 19;   { setting axis and pip colors together }

CONST { new April 95 for ScaleType }
  Linear_scales = 0;
  LinearLog_scales = 1; { x = linear, y = log }
  LinearLn_scales = 2; { x = linear, y = ln = log2 }
  LogLinear_scales = 3; { y = linear, x = log }
  LnLinear_scales = 4; { y = linear, x = ln = log2 }
  LogLog_scales = 5; { x = log, y = log }
  LnLn_scales = 6; { x = ln, y = ln }

TYPE
  vartype = (CONSTANT, SIMPLE, GENERAL, SUBSCRIPTED, BUILTIN, FUNCT);
  varvalue = RECORD
    CASE vartype OF
      CONSTANT: (cvalue: TkReal);
      SIMPLE:   (vptr: Realptr);
      GENERAL, SUBSCRIPTED: (aptr: Arrayptr; lbnd, ubnd : integer);
      BUILTIN:  (fint : integer);
      FUNCT:    (fptr : Funcptr);
    END;

TYPE { For dictionary elements in TkEval: Private }
  dictptr = ^ dictent;
  dictent = RECORD
    name : string;
    sort : vartype;
    valu : varvalue;
    note : boolean;
    next : dictptr
  END;

CONST HASHMAX = 11;

TYPE  { For dictionary in TkEval: Private }
  PMainDict = ^MainDictRec;
  MainDictRec = RECORD
    hashtab : ARRAY [1..HASHMAX] OF dictptr;
    hashold : ARRAY [1..HASHMAX] OF dictptr;
  END;

CONST { for SetSubGraphParam }
  nxi_gp     = 1;   { no. of intervals }
  nyi_gp     = 2;
  nzi_gp     = 3;
  nSubxi_gp  = 101;   { no. of intervals }
  nSubyi_gp  = 102;
  nSubzi_gp  = 103;
  mlf_gp     = 4;   { margin size in pixels }
  mrt_gp     = 5;
  mbt_gp     = 6;
  mtp_gp     = 7;
  labeldx_gp = 11;  { Graph label displacement in pixels }
  labeldy_gp = 12;
  xfield_gp  = 13;  { label field size in no. of characters }
  yfield_gp  = 14;
  zfield_gp  = 15;
  xpipm_gp   = 16;  { pip mode: 2 - centre, 1 - upwards or rightwards, }
  ypipm_gp   = 17;  {           0 - downwards or leftwards; }
  zpipm_gp   = 18;  { *** NEW: 0-Invisible, 1-Inside, 2-centre, 3-outside }
  xpipl_gp   = 19;  { pip length: percentages of length of axis }
  ypipl_gp   = 20;  { *** NEW: percentages of height of cell }
  zpipl_gp   = 21;
  xSubpipl_gp= 119; { sub pip length: percentages of length of axis }
  ySubpipl_gp= 120; { *** NEW: percentages of height of cell }
  zSubpipl_gp= 121;
  setScalesMode_gp = 130;
  axesType_gp = 131;
  AutoNiceValues_gp = 132;
  Custom3DZoom_gp = 133;
  Auto3DOrigin_gp = 134;
  Projected3DOriginX_gp = 135;  { percentage of 'view rect' }
  Projected3DOriginY_gp = 136;  { percentage of 'view rect' }
  SymbolHeight_gp = 137; { in pixels }
  { new bits on June 94 }
  GridOption_gp = 140;
  GridStyle1_gp = 141;
  GridStyle2_gp = 142;
  AutoMargins_gp = 143;
  AxisWidth_gp = 144;
  AxisLineStyle_gp = 145;
  GridWidth1_gp = 146;
  GridWidth2_gp = 147;
  UsePointColor_gp = 148;
  ScaleType_gp = 149;

CONST { symbols used in TkGraph.PlotSymbol, etc. }
  Blank_symbol = 0;
  Dot_symbol = 1;
  Plus_symbol = 2;
  Xcross_symbol = 3;
  Square_symbol = 4;
  Triangle_symbol = 5;
  Circle_symbol = 6;
  FilledSquare_symbol = 7;
  FilledTriangle_symbol = 8;
  FilledCircle_symbol = 9;
  NumSymbols = 10;
  { add const by 10 gives joined symbol version in TkDraw (Draw_data) }

{-- For Math error code ----------------------------------------------------}
VAR MathError: integer;
    MathTolerance: TkReal;

{-- ODE solver -------------------------------------------------------------}
CONST MaxODE = 8;
      DepVar_ODE = -1;
      DepVarStep_ODE = -2;
      DepVarMinStep_ODE = -3;

{-- Main list of functions and procedures ----------------------------------}
FUNCTION  TkNewArray1(n: integer): PArray;
  { 1D array, index from 1 to n }
FUNCTION  TkNewArray2(m, n: integer): PArray;
  { 2D array, first index from 1 to m, second index from 1 to n }
FUNCTION  TkNewArray3(l, m, n: integer): PArray;
  { 3D array, 1st index from 1 to l, 2nd from 1 to m, 3rd from 1 to n }
PROCEDURE TkDisposeArray(p: PArray);
FUNCTION  TkGetArray1(p: PArray; n: integer): TkReal;
FUNCTION  TkGetArray2(p: PArray; m, n: integer): TkReal;
FUNCTION  TkGetArray3(p: PArray; l, m, n: integer): TkReal;
PROCEDURE TkSetArray1(p: PArray; n: integer; val: TkReal);
PROCEDURE TkSetArray2(p: PArray; m, n: integer; val: TkReal);
PROCEDURE TkSetArray3(p: PArray; l, m, n: integer; val: TkReal);
FUNCTION  TkGetArrayAddress(p: PArray): Pointer;
FUNCTION  TkRGB(rcol, gcol, bcol: integer; UseSolidColor: WordBool): TkColor;
FUNCTION  RaisePower(value, power : TkReal) : TkReal;
FUNCTION  Sgn(x : TkReal) : integer;
FUNCTION  Tan(x : TkReal) : TkReal;
FUNCTION  Arccos(x : TkReal) : TkReal;
FUNCTION  Arcsin(x : TkReal) : TkReal;

{-- Common string functions and others (Pascal specific) -------------------}
FUNCTION MakePChar(VAR str: string): PChar; 
PROCEDURE Trim(VAR str: string);
FUNCTION IntToShortStr(i: Longint): string;
FUNCTION RealToStr(r: TkReal; dec: integer): string;
FUNCTION TkLowMemory: WordBool;
PROCEDURE Debug(str: string);
PROCEDURE SetDebugMode(switch: Boolean);
FUNCTION TkGetVersion: integer;
FUNCTION Timer: longint;
FUNCTION GetDCSettings(option: integer; VAR Settings: DCSettings): boolean;
FUNCTION  TkNewBlock(n: WORD): Pointer;
PROCEDURE TkDisposeBlock(p: Pointer; n: Word);
FUNCTION TkFormatReal (rval: TkReal; field: integer; buffer: PChar): integer;
PROCEDURE TkGetColorSet(var buffer);

{-- Graph window specific --------------------------------------------------}
FUNCTION  TkOpenGraph(aparent: HWnd; X, Y, W, H: integer;
  dmode: integer): PkGraph;
PROCEDURE TkCloseGraph(gwnd: PkGraph);
PROCEDURE TkShowGraph(gwnd: PkGraph);
PROCEDURE TkHideGraph(gwnd: PkGraph);
FUNCTION TkGetGraphHWnd(gwnd: PkGraph): HWnd;
FUNCTION TkSetDrawingMode(gwnd: PkGraph; mode: integer): integer;
PROCEDURE TkSToMPStyle(gwnd: PkGraph; option: integer);
PROCEDURE TkSetMessageRoute(gwnd: PkGraph; route: HWnd);

{-- Global variables for the current DC and current font -------------------}
FUNCTION TkWindowWidth(gwnd: PkGraph): integer;
FUNCTION TkWindowHeight(gwnd: PkGraph): integer;
FUNCTION TkCellWidth(gwnd: PkGraph): integer;
FUNCTION TkCellHeight(gwnd: PkGraph): integer;
FUNCTION TkEndTextX(gwnd: PkGraph): integer;
FUNCTION TkEndTextY(gwnd: PkGraph): integer;

{-- Colors, font, writemodes and pen setting -------------------------------}
PROCEDURE TkTextColor(gwnd: PkGraph; clr: TkColor);
FUNCTION  TkTextJustified(gwnd: PkGraph; option: integer): integer;
PROCEDURE TkFont(gwnd: PkGraph; face, style, size: integer);
PROCEDURE TkFont2(gwnd: PkGraph; face, style, size, orient: integer);
PROCEDURE TkBackColor(gwnd: PkGraph; clr: TkColor);
PROCEDURE TkGraphColor(gwnd: PkGraph; clr: TkColor);
PROCEDURE TkWritemode(gwnd: PkGraph; writemode: integer);
PROCEDURE TkPen(gwnd: PkGraph; clr: TkColor; style, size: integer);
PROCEDURE TkDefaults(gwnd: PkGraph); { does not clear screen }
PROCEDURE TkClearWindow(gwnd: PkGraph);

{-- Drawing routines using pixel coordinates ------------------------------}
PROCEDURE TkGpFill(gwnd: PkGraph; x1, y1, x2, y2: integer; clr:
  TkColor; writemode: integer); { revert to original color }
PROCEDURE TkGpPixel(gwnd: PkGraph; x, y: integer);
PROCEDURE TkGpColorPixel(gwnd: PkGraph; x, y : integer; clr: TkColor);
PROCEDURE TkGpLine(gwnd: PkGraph; x1, y1, x2, y2: integer);
PROCEDURE TkGpRectangle(gwnd: PkGraph; x1, y1, x2, y2 : integer;
  fill: Wordbool); { unfilled rectangle }
PROCEDURE TkGpPoly(gwnd: PkGraph; n: integer; p: Coordptr; fill:
  Wordbool);
  { For a closed polygon, the starting point must be  }
  { specified twice in order to close the polygon ... }
PROCEDURE TkGpCircle(gwnd: PkGraph; x, y, r: integer; fill:
  Wordbool);
PROCEDURE TkGpText(gwnd: PkGraph; x, y: integer; Cline: PChar);

{-- Image manipulation - x,y pixel coords., Redraw/Dispatch/Scroll. --------}
FUNCTION  TkGetImage(gwnd: PkGraph; x1, y1, x2, y2: integer): TkBitmap;
PROCEDURE TkPutImage(gwnd: PkGraph; x, y: integer; b: TkBitmap;
  writemode: integer);
PROCEDURE TkClearImage(b: TkBitmap);
PROCEDURE TkScrollRect(gwnd: PkGraph; x1, y1, x2, y2, delx, dely:
  integer);
PROCEDURE TkScrollWindow(gwnd: PkGraph; delx, dely: integer);
PROCEDURE TkDispatchToWindow(gwnd: PkGraph);
PROCEDURE TkDispatchToRect(gwnd: PkGraph; x1, y1, x2, y2: integer);
FUNCTION  TkResize(gwnd: PkGraph; x, y, w, h: integer): WordBool;
PROCEDURE ShowAboutBox(hInst: THandle; name: PChar);

{-- mouse and messages -----------------------------------------------------}
FUNCTION  TkSetMouseShape(gwnd: PkGraph; AShape: integer): integer;
PROCEDURE TkSetWaiting(gwnd: PkGraph; switchOn: Wordbool);
PROCEDURE TkGetPaintInfo(gwnd: PkGraph; VAR theInfo: TPaintStruct);
PROCEDURE TkGetMousePos(gwnd: PkGraph; VAR thePos: TPoint);

{-- User-defined drawing settings ------------------------------------------}
PROCEDURE TkSetRanges(gwnd: PkGraph; xlo, xhi, ylo, yhi: TkReal);
PROCEDURE TkSetRanges3(gwnd: PkGraph; xlo, xhi, ylo, yhi, zlo, zhi:
  TkReal);
PROCEDURE TkSetOrigin(gwnd: PkGraph; xorig, yorig: TkReal);
PROCEDURE TkSetOrigin3(gwnd: PkGraph; xorig, yorig, zorig: TkReal);
PROCEDURE TkSetViewangles(gwnd: PkGraph; th, ph: TkReal);
PROCEDURE TkSetView2D(gwnd: PkGraph);
PROCEDURE TkSetView3D(gwnd: PkGraph);
PROCEDURE TkSetColors(gwnd: PkGraph; axiscolor, pipcolor,
  labcolor, linecolor: TkColor);
PROCEDURE TkSetGraphColor(gwnd: PkGraph; index: integer; clr: TkColor);
FUNCTION TkGetGraphColor(gwnd: PkGraph; index: integer): TkColor;
PROCEDURE TkSetMargins(gwnd: PkGraph; left, top, right, bottom:
  integer);
PROCEDURE TkSetGraphParam(gwnd: PkGraph; item, ival: integer);
PROCEDURE TkGetLabelSettings(gwnd: PkGraph; labelType: integer;
  VAR result: LabelSettings);
PROCEDURE TkSetLabelSettings(gwnd: PkGraph; labelType: integer;
  theSettings: PLabelSettings);
FUNCTION TkGetGraphParam(gwnd: PkGraph; item: integer): integer;
PROCEDURE TkSetTitleString(gwnd: PkGraph; titleType: integer; s: PChar);
FUNCTION TkGetTitleString(gwnd: PkGraph; titleType: integer; buffer: PChar;
  size: integer): integer; { maximum size -1 characters recieved }

{-- Axes labelling, etc ----------------------------------------------------}
PROCEDURE TkSetScales(gwnd: PkGraph);
PROCEDURE TkSetScalesAbs(gwnd: PkGraph);
PROCEDURE TkDrawBorder(gwnd: PkGraph);
PROCEDURE TkDrawAxes(gwnd: PkGraph);
PROCEDURE TkDrawBox(gwnd: PkGraph);
PROCEDURE TkDrawGrid(gwnd: PkGraph);
PROCEDURE TkLabelAxes(gwnd: PkGraph);

{-- 2d and 3d conversions. User <-> screen coords --------------------------}
FUNCTION TkUserToWindowX(gwnd: PkGraph; x, y: TkReal): integer;
FUNCTION TkUserToWindowY(gwnd: PkGraph; x, y: TkReal): integer;
FUNCTION TkWindowToUserX(gwnd: PkGraph; p, q: integer): TkReal;
FUNCTION TkWindowToUserY(gwnd: PkGraph; p, q: integer): TkReal;
FUNCTION TkUserToWindowX3(gwnd: PkGraph; x, y, z: TkReal): integer;
FUNCTION TkUserToWindowY3(gwnd: PkGraph; x, y, z: TkReal): integer;

{-- Basic moves and draws --------------------------------------------------}
PROCEDURE TkMoveabs(gwnd: PkGraph; x, y: TkReal);
PROCEDURE TkMoverel(gwnd: PkGraph; x, y: TkReal);
PROCEDURE TkMoveabs3(gwnd: PkGraph; x, y, z: TkReal);
PROCEDURE TkMoverel3(gwnd: PkGraph; x, y, z: TkReal);
PROCEDURE TkDrawabs(gwnd: PkGraph; x, y: TkReal);
PROCEDURE TkDrawrel(gwnd: PkGraph; x, y: TkReal);
PROCEDURE TkDrawabs3(gwnd: PkGraph; x, y, z: TkReal);
PROCEDURE TkDrawrel3(gwnd: PkGraph; x, y, z: TkReal);

{-- Labels, symbols and cross-wires ----------------------------------------}
PROCEDURE TkGraphLabel(gwnd: PkGraph; x, y: TkReal; s: PChar);
PROCEDURE TkGraphLabel3(gwnd: PkGraph; x, y, z: TkReal; s: PChar);
PROCEDURE TkPlotSymbol(gwnd: PkGraph; x, y: TkReal; symbol: integer);
PROCEDURE TkPlotSymbol3(gwnd: PkGraph; x, y, z: TkReal; symbol: integer);
PROCEDURE TkCrossWires(gwnd: PkGraph; x, y: TkReal);
PROCEDURE TkCrossWires3(gwnd: PkGraph; x, y, z: TkReal);
PROCEDURE TkErrorBar(gwnd: PkGraph; x, y, errx, erry: TkReal;
  symbol: integer);
PROCEDURE TkGraphTitle(gwnd: PkGraph; titleType: integer; s: PChar);

{-- Draw/fill polygon in 2d & 3d -------------------------------------------}
FUNCTION TkDrawPolygon(gwnd: PkGraph; x, y: PArray; n:
  integer): integer;
  { x, y must be one dimensional array with at least n elements }
FUNCTION TkFillPolygon(gwnd: PkGraph; x, y: PArray; n:
  integer): integer;
FUNCTION TkDrawPolygon3(gwnd: PkGraph; x, y, z: PArray;
  n: integer): integer;
  { x, y, z must be one dimensional array with at least n elements }
FUNCTION TkFillPolygon3(gwnd: PkGraph; x, y, z: PArray;
  n: integer): integer;

{-- Draw curve based on expression -----------------------------------------}
FUNCTION TkDrawCurve1(gwnd: PkGraph; cstr, cvar: PChar; xl, xh:
  TkReal; n: integer; clr: TkColor; axismode : Axismodeoption): integer;
  { Draw curve y=f(x) based on (n-1) line segments over range [xl,xh].}
FUNCTION TkDrawCurve2(gwnd: PkGraph; cstrx, cstry, cvar: PChar;
  sl, sh: TkReal; n: integer; clr: TkColor; axismode: Axismodeoption):
  integer;
  { draw parameterised curves x=cstrx(s), y=cstry(s) }
FUNCTION TkDrawCurve3(gwnd: PkGraph; cstrx, cstry, cstrz, cvar:
  PChar; sl, sh: TkReal; n: integer; clr: TkColor; axismode: Axismodeoption):
  integer;
  { draw parameterised curves x=cstrx(s), y=cstry(s), z=cstrz(s) }

{-- Draw curve based on user-defined function ------------------------------}
FUNCTION TkDrawCurveFunc1(gwnd: PkGraph; f: Funcptr; xl, xh:
  TkReal; n: integer; clr: TkColor; axismode : Axismodeoption): integer;
  { Draw curve y=f(x) based on (n-1) line segments over range [xl,xh].}
FUNCTION TkDrawCurveFunc2(gwnd: PkGraph; f: Funcptr; VAR x, y: TkReal;
  sl, sh: TkReal; n: integer; clr: TkColor; axismode: Axismodeoption):
  integer;
  { draw parameterised curves x=cstrx(s), y=cstry(s) }
FUNCTION TkDrawCurveFunc3(gwnd: PkGraph; f: Funcptr; VAR x, y, z:
  TkReal; sl, sh: TkReal; n: integer; clr: TkColor; axismode:
  Axismodeoption): integer;
  { draw parameterised curves x=cstrx(s), y=cstry(s), z=cstrz(s) }

{-- Sample functions f(x), f(x,y) and f(x,y,z) -----------------------------}
FUNCTION TkSampleFunction1(cstr, cvar: PChar; xl, xh: TkReal; u, a: PArray;
  n: integer): integer;
  { u, a each must be one dimensional array with more than ni elements, or
    they may be nil (no corresponding output) }
FUNCTION TkSampleFunction2(cstr, cvar1, cvar2: PChar; xl, xh, yl, yh: TkReal;
  u, v, a: PArray; ni, nj: integer): integer;
  { u, v must be one dimensional arrays with more than ni and nj elements
    respectively, a must be a two dimensional array with at least ni
    elements for 1st index and nj elements for 2nd index.
    [IMPORTANT NOTE]  in the old routine Sample_function2, ni corresponds to
    dimension y, nj corresponds to dimension x.  See toolkit (DOS) manual. }
FUNCTION TkSampleFunction3(cstr, cvar1, cvar2, cvar3: PChar; xl, xh, yl, yh,
  zl, zh: TkReal; u, v, w, a: PArray; ni, nj, nk: integer): integer;
  { u, v, w must be one dimensional arrays with at least ni, nj, nk
  elements respectively, a must be a three dimension array with enough
  storage size as usual.
  [IMPORTANT NOTE]  in the old routine Sample_function3, ni corresponds to
  dimension z, nj corresponds to dimension y, and nk corresponds to
  dimension x.  See toolkit (DOS) manual. }

{-- Draw data in arrays as points ------------------------------------------}
FUNCTION TkDrawArray1(gwnd: PkGraph; u, v: PArray; n: integer; opt:
  integer; clr: TkColor; axismode: Axismodeoption): integer;
  { Draw data yi against xi; u, v must be one dimensional arrays with at
    least ni elements. }
FUNCTION TkDrawArray2(gwnd: PkGraph; u, v, a: PArray; ni, nj: integer;
  opt: integer; clr: TkColor; axismode: Axismodeoption): integer;
  { Draw data z[xi,yj]; see similar remark in TkSampleFunction2. }
FUNCTION TkDrawArray3(gwnd: PkGraph; u, v, w: PArray; n: integer;
  opt: integer; clr: TkColor; axismode: Axismodeoption): integer;
  { Draw data  xi,yi,zi;  u, v, w must be one dimensional arrays with at
    least n elements. }

{-- Histogram --------------------------------------------------------------}
FUNCTION TkDrawHistogram1(gwnd: PkGraph; u, v: PArray; n: integer;
  width, disp: TkReal; clr: TkColor; axismode: Axismodeoption): integer;
  { u, v must be one dimensional arrays with at least n elements. }
FUNCTION TkDrawHistogram2(gwnd: PkGraph; u, v, a: PArray; ni, nj:
  integer; width, depth: TkReal; clr: TkColor; axismode: Axismodeoption):
  integer;
  { See TkSampleFunction2 for similar remark. }

{-- Contours, surfaces and revolutions -------------------------------------}
FUNCTION TkDrawContoursArray(gwnd: PkGraph; u, v, a: PArray; ni, nj,
  ncontours: integer; zl, zh: TkReal; opt: Drawobjectoption; clr: TkColor;
  axismode: Axismodeoption): integer;
  { Draw contours of surface  z[xi,yj], see also TkSampleFunction2. }
FUNCTION TkDrawContours(gwnd: PkGraph; cstr, cvar1, cvar2: PChar;
  xl, xh, yl, yh: TkReal; ni, nj, ncontours: integer; zl, zh: TkReal; opt:
  Drawobjectoption; clr: TkColor; axismode: Axismodeoption): integer;
FUNCTION TkDrawSurfaceArray(gwnd: PkGraph; u, v, a: PArray; ni, nj:
  integer; opt: Drawobjectoption; clr1, clr2: TkColor; axismode:
  Axismodeoption): integer;
  { Draw surface z[xi,yj], see also TkSampleFunction2. }
FUNCTION TkDrawSurface(gwnd: PkGraph; cstr, cvar1, cvar2: PChar;
  xl, xh, yl, yh: TkReal; ni, nj: integer; opt: Drawobjectoption; clr1,
  clr2: TkColor; axismode: Axismodeoption): integer;
FUNCTION TkDrawRevolutionArray(gwnd: PkGraph; z, r: PArray; n,
  nslice: integer; opt: Drawobjectoption; clr1, clr2: TkColor; axismode:
  Axismodeoption): integer;
  { z, r are one dimensional arrays of at least n elements. }
FUNCTION TkDrawRevolution(gwnd: PkGraph; cstr, cvar: PChar; zl,
  zh: TkReal; n, nslice: integer; opt: Drawobjectoption; clr1, clr2:
  TkColor; axismode: Axismodeoption): integer;

{-- TkMath: Zeroin, Fmin, Romb ---------------------------------------------}
FUNCTION TkFindRoot(a, b, tol: TkReal; cstr, cvar: PChar): TkReal;
FUNCTION TkFindmin(a, b, tol: TkReal; cstr, cvar: PChar): TkReal;
FUNCTION TkIntegrate(a, b: TkReal; n: integer; cstr, cvar: PChar): TkReal;
  { Integrate f(x) dx over the interval (a,b). (Romberg) }
  { An error estimate is returned in the variable 'error'. }

{-- Evaluate expressions ---------------------------------------------------}
FUNCTION TkEvaluate1(cstr, cvar: PChar; x: TkReal): TkReal;
FUNCTION TkEvaluate2(cstr, cvar1, cvar2: PChar; x, y: TkReal): TkReal;
FUNCTION TkEvaluate3(cstr, cvar1, cvar2, cvar3: PChar; x, y, z: TkReal):
  TkReal;
FUNCTION TkEvaluateEx0ByVar(gwnd: PkGraph; cstr: PChar; VAR result: TkReal):
  integer;
FUNCTION TkCompileEx0(gwnd: PkGraph; cstr: PChar): integer;

{-- Parameter list ---------------------------------------------------------}
PROCEDURE TkClearParaList(gwnd: PkGraph);
FUNCTION  TkAddParaList(gwnd: PkGraph; cstr: PChar; ivalue: TkReal): integer;
FUNCTION  TkSetParaValue(gwnd: PkGraph; cstr: PChar; ivalue: TkReal): integer;
FUNCTION  TkCompileParaList(gwnd: PkGraph): integer;
FUNCTION  TkGetErrorName(name: PChar; size: integer): integer;

{-- ODE solver -------------------------------------------------------------}
FUNCTION TkNewODE(n: integer; cdepvar: PChar; aber,reler: TkReal): PODEFuncs;
PROCEDURE TkDisposeODE(p: PODEFuncs);
FUNCTION TkSetODELine(p: PODEFuncs; index: integer; cvar, cexp: PChar):
  integer;
PROCEDURE TkSetODEValue(p: PODEFuncs; index: integer; newvalue: TkReal);
FUNCTION TkGetODEValue(p: PODEFuncs; index: integer): TkReal;
FUNCTION TkODESolve(p: PODEFuncs): integer;

{-- TkMath: Zeroin, Fmin, Romb (original routines ) ------------------------}
FUNCTION Zeroin(a, b, tol: TkReal; fp: Funcptr): TkReal;
FUNCTION Fmin(a, b, tol: TkReal; fp: Funcptr) : TkReal;
FUNCTION Romb(a, b: TkReal; n: integer; fp: Funcptr; VAR error: TkReal):
  TkReal;
PROCEDURE Rkf(n: integer; aber, reler: TkReal; VAR t, dt: TkReal;
  dtmin: TkReal; fp: Funcptr; y, yp: Arrayptr; VAR nleft: integer;
  VAR flag: boolean);
PROCEDURE Fft (f: Arrayptr; m: integer; sign: TkReal);

{-- Import from DLL --------------------------------------------------------}
IMPLEMENTATION

{ private link to DLL }
FUNCTION TkFindRootByVar(a, b, tol: TkReal; cstr, cvar: PChar; VAR result:
  TkReal): integer; far; external GRAPHDLL;
FUNCTION TkFindminByVar(a, b, tol: TkReal; cstr, cvar: PChar; VAR result:
  TkReal): integer; far; external GRAPHDLL;
FUNCTION TkIntegrateByVar(a, b: TkReal; n: integer; cstr, cvar: PChar;
  VAR result, errest: TkReal): integer; far; external GRAPHDLL;
FUNCTION TkEvaluate1ByVar(cstr, cvar: PChar; x: TkReal; VAR result: TkReal):
  integer; far; external GRAPHDLL;
FUNCTION TkEvaluate2ByVar(cstr, cvar1, cvar2: PChar; x, y: TkReal; VAR result:
  TkReal): integer; far; external GRAPHDLL;
FUNCTION TkEvaluate3ByVar(cstr, cvar1, cvar2, cvar3: PChar; x, y, z: TkReal;
  VAR result: TkReal): integer; far; external GRAPHDLL;
FUNCTION TkEvaluateEx0ByVar(gwnd: PkGraph; cstr: PChar; VAR result: TkReal):
  integer; external GRAPHDLL;
FUNCTION TkCompileEx0(gwnd: PkGraph; cstr: PChar): integer; external GRAPHDLL;
PROCEDURE TkGetODEValueByVar(p: PODEFuncs; index: integer; VAR val:
  TkReal); far; external GRAPHDLL;

{ link to DLL }
FUNCTION  TkNewArray1(n: integer): PArray; external GRAPHDLL;
FUNCTION  TkNewArray2(m, n: integer): PArray; external GRAPHDLL;
FUNCTION  TkNewArray3(l, m, n: integer): PArray; external GRAPHDLL;
PROCEDURE TkDisposeArray(p: PArray); external GRAPHDLL;
FUNCTION  TkGetArray1(p: PArray; n: integer): TkReal; external GRAPHDLL;
FUNCTION  TkGetArray2(p: PArray; m, n: integer): TkReal; external GRAPHDLL;
FUNCTION  TkGetArray3(p: PArray; l, m, n: integer): TkReal;
  external GRAPHDLL;
PROCEDURE TkSetArray1(p: PArray; n: integer; val: TkReal); 
  external GRAPHDLL;
PROCEDURE TkSetArray2(p: PArray; m, n: integer; val: TkReal);
  external GRAPHDLL;
PROCEDURE TkSetArray3(p: PArray; l, m, n: integer; val: TkReal);
  external GRAPHDLL;
FUNCTION  TkRGB(rcol, gcol, bcol: integer; UseSolidColor: WordBool): TkColor;
  external GRAPHDLL;
FUNCTION  TkGetArrayAddress(p: PArray): pointer; external GRAPHDLL;
FUNCTION  RaisePower(value, power : TkReal) : TkReal; external GRAPHDLL;
FUNCTION  Sgn(x : TkReal) : integer; external GRAPHDLL;
FUNCTION  Tan(x : TkReal) : TkReal; external GRAPHDLL;
FUNCTION  Arccos(x : TkReal) : TkReal; external GRAPHDLL;
FUNCTION  Arcsin(x : TkReal) : TkReal; external GRAPHDLL;
FUNCTION  MakePChar(VAR str: string): PChar; external GRAPHDLL;
PROCEDURE Trim(VAR str: string); external GRAPHDLL;
FUNCTION  IntToShortStr(i: Longint): string; external GRAPHDLL;
FUNCTION  RealToStr(r: TkReal; dec: integer): string; external GRAPHDLL;
FUNCTION  TkLowMemory: WordBool; external GRAPHDLL;
PROCEDURE Debug(str: string); external GRAPHDLL;
FUNCTION  TkGetVersion: integer; external GRAPHDLL;
FUNCTION  Timer: longint; external GRAPHDLL;
FUNCTION  GetDCSettings(option: integer; VAR Settings: DCSettings): boolean;
  external GRAPHDLL;
FUNCTION  TkNewBlock(n: WORD): Pointer; external GRAPHDLL;
PROCEDURE TkDisposeBlock(p: Pointer; n: Word); external GRAPHDLL;
FUNCTION TkFormatReal (rval: TkReal; field: integer; buffer: PChar):
  integer; external GRAPHDLL;
PROCEDURE TkGetColorSet(var buffer); external GRAPHDLL;
PROCEDURE SetDebugMode(switch: Boolean); external GRAPHDLL;
FUNCTION  TkOpenGraph(aparent: HWnd; X, Y, W, H: integer;
  dmode: integer): PkGraph; external GRAPHDLL;
PROCEDURE TkCloseGraph(gwnd: PkGraph); external GRAPHDLL;
PROCEDURE TkShowGraph(gwnd: PkGraph); external GRAPHDLL;
PROCEDURE TkHideGraph(gwnd: PkGraph); external GRAPHDLL;
FUNCTION TkGetGraphHWnd(gwnd: PkGraph): HWnd; external GRAPHDLL;
FUNCTION  TkSetDrawingMode(gwnd: PkGraph; mode: integer): integer;
  external GRAPHDLL;
PROCEDURE TkSToMPStyle(gwnd: PkGraph; option: integer); external GRAPHDLL;
PROCEDURE TkSetMessageRoute(gwnd: PkGraph; route: HWnd); external GRAPHDLL;
FUNCTION  TkWindowWidth(gwnd: PkGraph): integer; external GRAPHDLL;
FUNCTION  TkWindowHeight(gwnd: PkGraph): integer; external GRAPHDLL;
FUNCTION  TkCellWidth(gwnd: PkGraph): integer; external GRAPHDLL;
FUNCTION  TkCellHeight(gwnd: PkGraph): integer; external GRAPHDLL;
FUNCTION  TkEndTextX(gwnd: PkGraph): integer; external GRAPHDLL;
FUNCTION  TkEndTextY(gwnd: PkGraph): integer; external GRAPHDLL;
PROCEDURE TkTextColor(gwnd: PkGraph; clr: TkColor); external GRAPHDLL;
FUNCTION  TkTextJustified(gwnd: PkGraph; option: integer): integer;
  external GRAPHDLL;
PROCEDURE TkFont(gwnd: PkGraph; face, style, size: integer);
  external GRAPHDLL;
PROCEDURE TkFont2(gwnd: PkGraph; face, style, size, orient: integer);
  external GRAPHDLL;
PROCEDURE TkBackColor(gwnd: PkGraph; clr: TkColor); external GRAPHDLL;
PROCEDURE TkGraphColor(gwnd: PkGraph; clr: TkColor); external GRAPHDLL;
PROCEDURE TkWritemode(gwnd: PkGraph; writemode: integer);
  external GRAPHDLL;
PROCEDURE TkPen(gwnd: PkGraph; clr: TkColor; style, size: integer);
  external GRAPHDLL;
PROCEDURE TkDefaults(gwnd: PkGraph); external GRAPHDLL;
PROCEDURE TkClearWindow(gwnd: PkGraph); external GRAPHDLL;
PROCEDURE TkGpFill(gwnd: PkGraph; x1, y1, x2, y2: integer; clr:
  TkColor; writemode: integer); external GRAPHDLL;
PROCEDURE TkGpPixel(gwnd: PkGraph; x, y: integer); external GRAPHDLL;
PROCEDURE TkGpColorPixel(gwnd: PkGraph; x, y : integer; clr: TkColor);
  external GRAPHDLL;
PROCEDURE TkGpLine(gwnd: PkGraph; x1, y1, x2, y2: integer);
  external GRAPHDLL;
PROCEDURE TkGpRectangle(gwnd: PkGraph; x1, y1, x2, y2 : integer;
  fill: Wordbool); external GRAPHDLL;
PROCEDURE TkGpPoly(gwnd: PkGraph; n: integer; p: Coordptr; fill:
  Wordbool); external GRAPHDLL;
PROCEDURE TkGpCircle(gwnd: PkGraph; x, y, r: integer; fill:
  Wordbool); external GRAPHDLL;
PROCEDURE TkGpText(gwnd: PkGraph; x, y: integer; Cline: PChar);
  external GRAPHDLL;
FUNCTION TkGetImage(gwnd: PkGraph; x1, y1, x2, y2: integer): TkBitmap;
  external GRAPHDLL;
PROCEDURE TkPutImage(gwnd: PkGraph; x, y: integer; b: TkBitmap;
  writemode: integer); external GRAPHDLL;
PROCEDURE TkClearImage(b: TkBitmap); external GRAPHDLL;
PROCEDURE TkScrollRect(gwnd: PkGraph; x1, y1, x2, y2, delx, dely:
  integer); external GRAPHDLL;
PROCEDURE TkScrollWindow(gwnd: PkGraph; delx, dely: integer);
  external GRAPHDLL;
PROCEDURE TkDispatchToWindow(gwnd: PkGraph); external GRAPHDLL;
PROCEDURE TkDispatchToRect(gwnd: PkGraph; x1, y1, x2, y2: integer);
  external GRAPHDLL;
FUNCTION  TkResize(gwnd: PkGraph; x, y, w, h: integer): WordBool;
  external GRAPHDLL;
PROCEDURE ShowAboutBox(hInst: THandle; name: PChar); external GRAPHDLL;
FUNCTION  TkSetMouseShape(gwnd: PkGraph; AShape: integer): integer;
  external GRAPHDLL;
PROCEDURE TkSetWaiting(gwnd: PkGraph; switchOn: Wordbool); external GRAPHDLL;
PROCEDURE TkGetPaintInfo(gwnd: PkGraph; VAR theInfo: TPaintStruct);
  external GRAPHDLL;
PROCEDURE TkGetMousePos(gwnd: PkGraph; VAR thePos: TPoint); external GRAPHDLL;
PROCEDURE TkSetRanges(gwnd: PkGraph; xlo, xhi, ylo, yhi: TkReal);
  external GRAPHDLL;
PROCEDURE TkSetRanges3(gwnd: PkGraph; xlo, xhi, ylo, yhi, zlo, zhi:
  TkReal); external GRAPHDLL;
PROCEDURE TkSetOrigin(gwnd: PkGraph; xorig, yorig: TkReal);
  external GRAPHDLL;
PROCEDURE TkSetOrigin3(gwnd: PkGraph; xorig, yorig, zorig: TkReal);
  external GRAPHDLL;
PROCEDURE TkSetViewangles(gwnd: PkGraph; th, ph: TkReal);
  external GRAPHDLL;
PROCEDURE TkSetView2D(gwnd: PkGraph); external GRAPHDLL;
PROCEDURE TkSetView3D(gwnd: PkGraph); external GRAPHDLL;
PROCEDURE TkSetColors(gwnd: PkGraph; axiscolor, pipcolor,
  labcolor, linecolor: TkColor); external GRAPHDLL;
PROCEDURE TkSetGraphColor(gwnd: PkGraph; index: integer; clr: TkColor);
  external GRAPHDLL;
FUNCTION TkGetGraphColor(gwnd: PkGraph; index: integer): TkColor;
  external GRAPHDLL;
PROCEDURE TkSetMargins(gwnd: PkGraph; left, top, right, bottom:
  integer); external GRAPHDLL;
PROCEDURE TkSetGraphParam(gwnd: PkGraph; item, ival: integer);
  external GRAPHDLL;
PROCEDURE TkGetLabelSettings(gwnd: PkGraph; labelType: integer;
  VAR result: LabelSettings);
  external GRAPHDLL;
PROCEDURE TkSetLabelSettings(gwnd: PkGraph; labelType: integer;
  theSettings: PLabelSettings);
  external GRAPHDLL;
FUNCTION TkGetGraphParam(gwnd: PkGraph; item: integer): integer;
  external GRAPHDLL;
PROCEDURE TkSetTitleString(gwnd: PkGraph; titleType: integer; s: PChar);
  external GRAPHDLL;
FUNCTION TkGetTitleString(gwnd: PkGraph; titleType: integer; buffer: PChar;
  size: integer): integer; { maximum size -1 characters recieved }
  external GRAPHDLL;
PROCEDURE TkSetScales(gwnd: PkGraph); external GRAPHDLL;
PROCEDURE TkSetScalesAbs(gwnd: PkGraph); external GRAPHDLL;
PROCEDURE TkDrawBorder(gwnd: PkGraph); external GRAPHDLL;
PROCEDURE TkDrawAxes(gwnd: PkGraph); external GRAPHDLL;
PROCEDURE TkDrawBox(gwnd: PkGraph); external GRAPHDLL;
PROCEDURE TkDrawGrid(gwnd: PkGraph); external GRAPHDLL;
PROCEDURE TkLabelAxes(gwnd: PkGraph); external GRAPHDLL;
FUNCTION  TkUserToWindowX(gwnd: PkGraph; x, y: TkReal): integer;
  external GRAPHDLL;
FUNCTION  TkUserToWindowY(gwnd: PkGraph; x, y: TkReal): integer;
  external GRAPHDLL;
FUNCTION  TkWindowToUserX(gwnd: PkGraph; p, q: integer): TkReal;
  external GRAPHDLL;
FUNCTION  TkWindowToUserY(gwnd: PkGraph; p, q: integer): TkReal;
  external GRAPHDLL;
FUNCTION  TkUserToWindowX3(gwnd: PkGraph; x, y, z: TkReal): integer;
  external GRAPHDLL;
FUNCTION  TkUserToWindowY3(gwnd: PkGraph; x, y, z: TkReal): integer;
  external GRAPHDLL;
PROCEDURE TkMoveabs(gwnd: PkGraph; x, y: TkReal); external GRAPHDLL;
PROCEDURE TkMoverel(gwnd: PkGraph; x, y: TkReal); external GRAPHDLL;
PROCEDURE TkMoveabs3(gwnd: PkGraph; x, y, z: TkReal); external GRAPHDLL;
PROCEDURE TkMoverel3(gwnd: PkGraph; x, y, z: TkReal); external GRAPHDLL;
PROCEDURE TkDrawabs(gwnd: PkGraph; x, y: TkReal); external GRAPHDLL;
PROCEDURE TkDrawrel(gwnd: PkGraph; x, y: TkReal); external GRAPHDLL;
PROCEDURE TkDrawabs3(gwnd: PkGraph; x, y, z: TkReal); external GRAPHDLL;
PROCEDURE TkDrawrel3(gwnd: PkGraph; x, y, z: TkReal); external GRAPHDLL;
PROCEDURE TkGraphlabel(gwnd: PkGraph; x, y: TkReal; s: PChar);
  external GRAPHDLL name 'TkGraphlabel';
PROCEDURE TkGraphlabel3(gwnd: PkGraph; x, y, z: TkReal; s: PChar);
  external GRAPHDLL;
PROCEDURE TkPlotSymbol(gwnd: PkGraph; x, y: TkReal; symbol: integer);
  external GRAPHDLL;
PROCEDURE TkPlotSymbol3(gwnd: PkGraph; x, y, z: TkReal; symbol:
  integer); external GRAPHDLL;
PROCEDURE TkCrossWires(gwnd: PkGraph; x, y: TkReal); external GRAPHDLL;
PROCEDURE TkCrossWires3(gwnd: PkGraph; x, y, z: TkReal); external GRAPHDLL;
PROCEDURE TkErrorBar(gwnd: PkGraph; x, y, errx, erry: TkReal;
  symbol: integer); external GRAPHDLL;
PROCEDURE TkGraphTitle(gwnd: PkGraph; titleType: integer; s: PChar);
  external GRAPHDLL;
FUNCTION  TkDrawPolygon(gwnd: PkGraph; x, y: PArray; n:
  integer): integer; external GRAPHDLL;
FUNCTION  TkFillPolygon(gwnd: PkGraph; x, y: PArray; n:
  integer): integer; external GRAPHDLL;
FUNCTION  TkDrawPolygon3(gwnd: PkGraph; x, y, z: PArray;
  n: integer): integer; external GRAPHDLL;
FUNCTION  TkFillPolygon3(gwnd: PkGraph; x, y, z: PArray;
  n: integer): integer; external GRAPHDLL;
FUNCTION  TkDrawCurve1(gwnd: PkGraph; cstr, cvar: PChar; xl, xh:
  TkReal; n: integer; clr: TkColor; axismode : Axismodeoption): integer;
  external GRAPHDLL;
FUNCTION  TkDrawCurve2(gwnd: PkGraph; cstrx, cstry, cvar: PChar;
  sl, sh: TkReal; n: integer; clr: TkColor; axismode: Axismodeoption):
  integer; external GRAPHDLL;
FUNCTION  TkDrawCurve3(gwnd: PkGraph; cstrx, cstry, cstrz, cvar:
  PChar; sl, sh: TkReal; n: integer; clr: TkColor; axismode: Axismodeoption):
  integer; external GRAPHDLL;
FUNCTION TkDrawCurveFunc1(gwnd: PkGraph; f: Funcptr; xl, xh:
  TkReal; n: integer; clr: TkColor; axismode : Axismodeoption): integer;
  external GRAPHDLL;
FUNCTION TkDrawCurveFunc2(gwnd: PkGraph; f: Funcptr; VAR x, y: TkReal;
  sl, sh: TkReal; n: integer; clr: TkColor; axismode: Axismodeoption):
  integer; external GRAPHDLL;
FUNCTION TkDrawCurveFunc3(gwnd: PkGraph; f: Funcptr; VAR x, y, z:
  TkReal; sl, sh: TkReal; n: integer; clr: TkColor; axismode:
  Axismodeoption): integer; external GRAPHDLL;
FUNCTION  TkSampleFunction1(cstr, cvar: PChar; xl, xh: TkReal; u, a: PArray;
  n: integer): integer; external GRAPHDLL;
FUNCTION  TkSampleFunction2(cstr, cvar1, cvar2: PChar; xl, xh, yl, yh: TkReal;
  u, v, a: PArray; ni, nj: integer): integer; external GRAPHDLL;
FUNCTION  TkSampleFunction3(cstr, cvar1, cvar2, cvar3: PChar; xl, xh, yl, yh,
  zl, zh: TkReal; u, v, w, a: PArray; ni, nj, nk: integer): integer;
  external GRAPHDLL;
FUNCTION  TkDrawArray1(gwnd: PkGraph; u, v: PArray; n: integer; opt:
  integer; clr: TkColor; axismode: Axismodeoption): integer;
  external GRAPHDLL;
FUNCTION  TkDrawArray2(gwnd: PkGraph; u, v, a: PArray; ni, nj: integer;
  opt: integer; clr: TkColor; axismode: Axismodeoption): integer;
  external GRAPHDLL;
FUNCTION  TkDrawArray3(gwnd: PkGraph; u, v, w: PArray; n: integer;
  opt: integer; clr: TkColor; axismode: Axismodeoption): integer;
  external GRAPHDLL;
FUNCTION  TkDrawHistogram1(gwnd: PkGraph; u, v: PArray; n: integer;
  width, disp: TkReal; clr: TkColor; axismode: Axismodeoption): integer;
  external GRAPHDLL;
FUNCTION  TkDrawHistogram2(gwnd: PkGraph; u, v, a: PArray; ni, nj:
  integer; width, depth: TkReal; clr: TkColor; axismode: Axismodeoption):
  integer; external GRAPHDLL;
FUNCTION  TkDrawContoursArray(gwnd: PkGraph; u, v, a: PArray; ni, nj,
  ncontours: integer; zl, zh: TkReal; opt: Drawobjectoption; clr: TkColor;
  axismode: Axismodeoption): integer; external GRAPHDLL;
FUNCTION  TkDrawContours(gwnd: PkGraph; cstr, cvar1, cvar2: PChar;
  xl, xh, yl, yh: TkReal; ni, nj, ncontours: integer; zl, zh: TkReal; opt:
  Drawobjectoption; clr: TkColor; axismode: Axismodeoption): integer;
  external GRAPHDLL;
FUNCTION  TkDrawSurfaceArray(gwnd: PkGraph; u, v, a: PArray; ni, nj:
  integer; opt: Drawobjectoption; clr1, clr2: TkColor; axismode:
  Axismodeoption): integer; external GRAPHDLL;
FUNCTION  TkDrawSurface(gwnd: PkGraph; cstr, cvar1, cvar2: PChar;
  xl, xh, yl, yh: TkReal; ni, nj: integer; opt: Drawobjectoption; clr1,
  clr2: TkColor; axismode: Axismodeoption): integer; external GRAPHDLL;
FUNCTION  TkDrawRevolutionArray(gwnd: PkGraph; z, r: PArray; n,
  nslice: integer; opt: Drawobjectoption; clr1, clr2: TkColor; axismode:
  Axismodeoption): integer; external GRAPHDLL;
FUNCTION  TkDrawRevolution(gwnd: PkGraph; cstr, cvar: PChar; zl,
  zh: TkReal; n, nslice: integer; opt: Drawobjectoption; clr1, clr2:
  TkColor; axismode: Axismodeoption): integer; external GRAPHDLL;
FUNCTION  TkNewODE(n: integer; cdepvar: PChar; aber, reler: TkReal):
  PODEFuncs; external GRAPHDLL;
PROCEDURE TkDisposeODE(p: PODEFuncs); external GRAPHDLL;
FUNCTION  TkSetODELine(p: PODEFuncs; index: integer; cvar, cexp: PChar):
  integer; external GRAPHDLL;
PROCEDURE TkSetODEValue(p: PODEFuncs; index: integer; newvalue: TkReal);
  external GRAPHDLL;
FUNCTION TkODESolve(p: PODEFuncs): integer; external GRAPHDLL;

{-- TkMath originals -------------------------------------------------------}
FUNCTION Zeroin(a, b, tol: TkReal; fp: Funcptr): TkReal; external GRAPHDLL;
FUNCTION Fmin(a, b, tol: TkReal; fp: Funcptr) : TkReal; external GRAPHDLL;
FUNCTION Romb(a, b: TkReal; n: integer; fp: Funcptr; VAR error: TkReal):
  TkReal; external GRAPHDLL;
PROCEDURE Rkf(n: integer; aber, reler: TkReal; VAR t, dt: TkReal;
  dtmin: TkReal; fp: Funcptr; y, yp: Arrayptr; VAR nleft: integer;
  VAR flag: boolean); external GRAPHDLL;
PROCEDURE Fft (f: Arrayptr; m: integer; sign: TkReal); external GRAPHDLL;

FUNCTION  TkFindRoot(a, b, tol: TkReal; cstr, cvar: PChar): TkReal;
VAR res: TkReal; { was result }
BEGIN
  MathError := TkFindRootByVar(a, b, tol, cstr, cvar, res);
  TkFindRoot := res
END; { TkFindRoot }

FUNCTION  TkFindmin(a, b, tol: TkReal; cstr, cvar: PChar): TkReal;
VAR res: TkReal; { ditto }
BEGIN
  MathError := TkFindminByVar(a, b, tol, cstr, cvar, res);
  TkFindmin := res
END; { TkFindmin }

FUNCTION  TkIntegrate(a, b: TkReal; n: integer; cstr, cvar: PChar): TkReal;
VAR res: TkReal; { ditto }
BEGIN
  MathError := TkIntegrateByVar(a, b, n, cstr, cvar, res, MathTolerance);
  TkIntegrate := res
END; { TkIntegrate }

FUNCTION  TkEvaluate1(cstr, cvar: PChar; x: TkReal): TkReal;
VAR res: TkReal; { ditto }
BEGIN
  MathError := TkEvaluate1ByVar(cstr, cvar, x, res);
  TkEvaluate1 := res;
END; { TkEvaluate1 }

FUNCTION  TkEvaluate2(cstr, cvar1, cvar2: PChar; x, y: TkReal): TkReal;
VAR res: TkReal; { ditto }
BEGIN
  MathError := TkEvaluate2ByVar(cstr, cvar1, cvar2, x, y, res);
  TkEvaluate2 := res;
END; { TkEvaluate2 }

FUNCTION  TkEvaluate3(cstr, cvar1, cvar2, cvar3: PChar; x, y, z: TkReal):
  TkReal;
VAR res: TkReal; { ditto }
BEGIN
  MathError := TkEvaluate3ByVar(cstr, cvar1, cvar2, cvar3, x, y, z, res);
  TkEvaluate3 := res;
END; { TkEvaluate3 }

PROCEDURE TkClearParaList(gwnd: PkGraph); external GRAPHDLL;
FUNCTION  TkAddParaList(gwnd: PkGraph; cstr: PChar; ivalue: TkReal):
  integer; external GRAPHDLL;
FUNCTION  TkSetParaValue(gwnd: PkGraph; cstr: PChar; ivalue: TkReal):
  integer; external GRAPHDLL;
FUNCTION  TkCompileParaList(gwnd: PkGraph): integer; external GRAPHDLL;
FUNCTION  TkGetErrorName(name: PChar; size: integer): integer; external GRAPHDLL;

FUNCTION TkGetODEValue(p: PODEFuncs; index: integer): TkReal;
VAR res: TkReal; { was result }
BEGIN
  TkGetODEValueByVar(p, index, res);
  TkGetODEValue := res;
END; { TkGetODEValue }


BEGIN
  {-- copy below from TkCore -----------------------------------------------}
  TkBLACK := TkRGB(0, 0, 0, true); { default colour combinations }
  TkBLUE := TkRGB(0, 0, 128, true);
  TkGREEN := TkRGB(0, 128, 0, true);
  TkCYAN := TkRGB(0, 120, 120, true);
  TkRED := TkRGB(132, 0, 0, true);
  TkMAGENTA := TkRGB(80, 0, 100, true);
  TkBROWN := TkRGB(128, 128, 0, true);
  TkLIGHTGRAY := TkRGB(192, 192, 192, true);
  TkDARKGRAY := TkRGB(128, 128, 128, true);
  TkLIGHTBLUE := TkRGB(84, 84, 255, true);
  TkLIGHTGREEN := TkRGB(84, 255, 84, true);
  TkLIGHTCYAN := TkRGB(20, 227, 227, true);
  TkLIGHTRED := TkRGB(255, 64, 64, true);
  TkLIGHTMAGENTA := TkRGB(135, 0, 171, true);
  TkYELLOW := TkRGB(228, 228, 76, true);
  TkWHITE := TkRGB(255, 255, 255, true);
  {-- copy above from TkCore -----------------------------------------------}
END.
